@@ -1,30 +1,31 @@
 # @package      hubzero-submit-distributor
 # @file         RemoteBatchPEGASUS.py
 # @author       Steven Clark <clarks@purdue.edu>
-# @copyright    Copyright (c) 2012-2014 HUBzero Foundation, LLC.
-# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
+# @copyright    Copyright (c) 2012-2015 HUBzero Foundation, LLC.
+# @license      http://opensource.org/licenses/MIT MIT
 #
-# Copyright (c) 2012-2014 HUBzero Foundation, LLC.
+# Copyright (c) 2012-2015 HUBzero Foundation, LLC.
 #
-# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
-# software: you can redistribute it and/or modify it under the terms of
-# the GNU Lesser General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any
-# later version.
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 #
-# HUBzero is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
 #
 # HUBzero is a registered trademark of HUBzero Foundation, LLC.
 #
-
 import os
 import re
 import logging
@@ -34,12 +35,15 @@ try:
 except:
    from hubzero.submit.DAX3 import *
 
-from hubzero.submit.JobOutput import JobOutput
+from hubzero.submit.LogMessage        import getLogJobIdMessage as getLogMessage 
+from hubzero.submit.ParameterTemplate import ParameterTemplate
+from hubzero.submit.JobOutput         import JobOutput
 
 class RemoteBatchPEGASUS:
    def __init__(self,
                 hubUserName,
                 hubUserId,
+                submitterClass,
                 runName,
                 localJobId,
                 instanceId,
@@ -53,14 +57,18 @@ class RemoteBatchPEGASUS:
                 arguments,
                 isMultiCoreRequest,
                 siteInfo,
+                submissionScriptsInfo,
                 managerInfo,
                 x509SubmitProxy,
+                sshIdentityPath,
                 gridsite,
+                wallTime,
                 timeHistoryLogs):
       self.logger            = logging.getLogger(__name__)
       self.jobOutput         = JobOutput()
       self.hubUserName       = hubUserName
       self.hubUserId         = hubUserId
+      self.submitterClass    = submitterClass
       self.runName           = runName
       self.localJobId        = localJobId
       self.instanceId        = instanceId
@@ -70,6 +78,7 @@ class RemoteBatchPEGASUS:
       self.pegasusVersion    = pegasusVersion
       self.pegasusHome       = pegasusHome
       self.appScriptName     = appScriptName
+      self.wallTime          = wallTime
       self.timestampStart    = timeHistoryLogs['timestampStart']
       self.timestampFinish   = timeHistoryLogs['timestampFinish']
       self.timeResults       = timeHistoryLogs['timeResults']
@@ -95,115 +104,45 @@ class RemoteBatchPEGASUS:
          self.arguments           = "--dax %s" % (self.daxPath)
          self.checkDaxExecutables = False
 
-      self.isMultiCoreRequest = isMultiCoreRequest
-      self.computationMode    = managerInfo['computationMode']
-      self.pegasusTemplates   = siteInfo['pegasusTemplates']
-      self.x509SubmitProxy    = x509SubmitProxy
-      self.gridsite           = gridsite
+      self.isMultiCoreRequest    = isMultiCoreRequest
+      self.computationMode       = managerInfo['computationMode']
+      self.pegasusTemplates      = siteInfo['pegasusTemplates']
+      self.submissionScriptsInfo = submissionScriptsInfo
+      self.x509SubmitProxy       = x509SubmitProxy
+      self.sshIdentityPath       = sshIdentityPath
+      self.gridsite              = gridsite
 
       self.nodeFileName = ""
       self.nodeList     = []
 
 
-   def __makeSerialTemplate(self):
-      return """#!/bin/sh
-# RemoteBatchPEGASUS:makeSerialTemplate
-#
-exitStatus=0
-
-mkdir -p INSTANCEDIRECTORY/work
-mkdir -p SCRATCHDIRECTORY/work
-mkdir -p INSTANCEDIRECTORY/InProcessResults
-
-PEGASUSSETUP
-
-TIMEPATH=
-for timePath in ${HOME}/bin/time /usr/bin/time /usr/local/bin/time /apps/submit/bin/time ; do
-   if [ -x ${timePath} ] ; then
-      TIMEPATH=${timePath}
-      break
-   fi
-done
-
-#Discarded Arguments      = DISCARDED
-#Not Recognized Arguments = NOTRECOGNIZED
-
-date +"%s" > TS_START
-
-${TIMEPATH} --format "Command exited with status %x\\nreal %e\\nuser %U\\nsys %S" -o TIME_RESULTS \\
-             EXECUTABLE --conf INSTANCEDIRECTORY/JOBID_INSTANCEID.pegasusrc \\
-             --sites GRIDSITE \\
-             --dir SCRATCHDIRECTORY \\
-             --relative-dir scratch \\
-             --relative-submit-dir work \\
-             --output local \\
-             ARGUMENTS \\
-             --submit < /dev/null
-exitStatus=$?
-
-date +"%s" > TS_FINISH
-
-exit ${exitStatus}
-"""
-
-
    def __buildSerialFile(self):
-      # setup regex's for the template
-      re_runName                = re.compile("RUNNAME")
-      re_jobId                  = re.compile("JOBID")
-      re_instanceId             = re.compile("INSTANCEID")
-      re_instanceDirectory      = re.compile("INSTANCEDIRECTORY")
-      re_scratchDirectory       = re.compile("SCRATCHDIRECTORY")
-      re_pegasusSetup           = re.compile("PEGASUSSETUP")
-      re_executable             = re.compile("EXECUTABLE")
-      re_gridsite               = re.compile("GRIDSITE")
-      re_arguments              = re.compile("ARGUMENTS")
-      re_discardedArguments     = re.compile("DISCARDED")
-      re_notRecognizedArguments = re.compile("NOTRECOGNIZED")
-      re_tsStart                = re.compile("TS_START")
-      re_tsFinish               = re.compile("TS_FINISH")
-      re_timeResults            = re.compile("TIME_RESULTS")
-      re_hubUserName            = re.compile("HUBUSERNAME")
-      re_hubUserId              = re.compile("HUBUSERID")
-
-      template = self.__makeSerialTemplate()
-
-      try:
-         pegasusSetupCommand = ". %s\nuse -e -r pegasus-%s" % (self.useSetup,self.pegasusVersion)
-         pegasusSetupCommandPath = "INSTANCEDIRECTORY/work/pegasusSetup.sh"
-         template = re_pegasusSetup.sub("cat > %s << EOF\n%s\nEOF\n. %s\n" % (pegasusSetupCommandPath,pegasusSetupCommand,
-                                                                              pegasusSetupCommandPath),template)
-      except:
-         template = re_pegasusSetup.sub("",template)
-
-      template = re_runName.sub(self.runName,template)
-      template = re_jobId.sub(self.localJobId,template)
-      template = re_instanceId.sub(self.instanceId,template)
-      template = re_instanceDirectory.sub(self.instanceDirectory,template)
-      template = re_scratchDirectory.sub(self.scratchDirectory,template)
-      template = re_executable.sub(self.executable,template)
-      template = re_gridsite.sub(self.gridsite,template)
-      template = re_tsStart.sub(self.timestampStart,template)
-      template = re_tsFinish.sub(self.timestampFinish,template)
-      template = re_timeResults.sub(self.timeResults,template)
+      rawSubmissionScript = self.submissionScriptsInfo.getSubmissionScript('Batch','PEGASUS','serial')
 
       notRecognizedArguments = []
       discardedArguments     = []
       auxiliaryArguments     = []
       if os.path.exists(self.pegasusTemplates['rc']):
-         fpRCTemplate = open(self.pegasusTemplates['rc'],'r')
-         if fpRCTemplate:
-            searchString = os.path.basename(self.executable) + '.arguments'
-            settings = fpRCTemplate.readlines()
-            for setting in settings:
+         try:
+            fpRCTemplate = open(self.pegasusTemplates['rc'],'r')
+            try:
+               searchString = os.path.basename(self.executable) + '.arguments'
+               settings = fpRCTemplate.readlines()
+            except (IOError,OSError):
+               self.logger.log(logging.ERROR,getLogMessage("%s could not be read" % (self.pegasusTemplates['rc'])))
+            else:
+               for setting in settings:
 #pegasus-plan.arguments = --nocleanup
-               if setting.count(searchString) > 0:
-                  try:
-                     parameter,value = setting.split('=')
-                     auxiliaryArguments.append(value.strip())
-                  except:
-                     pass
-            fpRCTemplate.close()
+                  if setting.count(searchString) > 0:
+                     try:
+                        parameter,value = setting.split('=')
+                        auxiliaryArguments.append(value.strip())
+                     except:
+                        pass
+            finally:
+               fpRCTemplate.close()
+         except (IOError,OSError):
+            self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (self.pegasusTemplates['rc'])))
 
       userArguments = self.arguments.split()
       location = 0
@@ -246,8 +185,11 @@ exit ${exitStatus}
                keepArgument = True
             elif userArguments[location] == '-n' or userArguments[location] == '--nocleanup':
                keepArgument = False
+            elif userArguments[location] == '--cleanup':
+               keepArgument = False
                discardedArguments.append(userArguments[location])
-            elif userArguments[location] == '-o' or userArguments[location] == '--output':
+            elif userArguments[location] == '-o' or \
+                 userArguments[location] == '--output' or userArguments[location] == '--output-site':
                keepArgument = False
                discardedArguments.append(userArguments[location])
             elif userArguments[location] == '-q' or userArguments[location] == '--quiet':
@@ -287,55 +229,98 @@ exit ${exitStatus}
          else:
             location += 1
 
+      substitutions = {}
+      substitutions["USESETUP"]          = self.useSetup
+      substitutions["PEGASUSVERSION"]    = self.pegasusVersion
+      substitutions["RUNNAME"]           = self.runName
+      substitutions["JOBID"]             = self.localJobId
+      substitutions["INSTANCEID"]        = self.instanceId
+      substitutions["INSTANCEDIRECTORY"] = self.instanceDirectory
+      substitutions["SCRATCHDIRECTORY"]  = self.scratchDirectory
+      substitutions["EXECUTABLE"]        = self.executable
+      substitutions["GRIDSITE"]          = self.gridsite
       arguments = ' '.join(auxiliaryArguments)
-      template  = re_arguments.sub(arguments.strip(),template)
+      substitutions["ARGUMENTS"]         = arguments.strip()
       arguments = ' '.join(discardedArguments)
-      template  = re_discardedArguments.sub(arguments.strip(),template)
+      substitutions["DISCARDED"]         = arguments.strip()
       arguments = ' '.join(notRecognizedArguments)
-      template  = re_notRecognizedArguments.sub(arguments.strip(),template)
-      template = re_hubUserName.sub(self.hubUserName,template)
-      template = re_hubUserId.sub(str(self.hubUserId),template)
+      substitutions["NOTRECOGNIZED"]     = arguments.strip()
+      substitutions["TS_START"]          = self.timestampStart
+      substitutions["TS_FINISH"]         = self.timestampFinish
+      substitutions["TIME_RESULTS"]      = self.timeResults
+      substitutions["HUBUSERNAME"]       = self.hubUserName
+      substitutions["HUBUSERID"]         = str(self.hubUserId)
 
-      return(template)
+      template = ParameterTemplate(rawSubmissionScript)
+      try:
+         submissionScript = template.substitute_recur(substitutions)
+      except KeyError,err:
+         submissionScript = ""
+         self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@%s\n" % (err[0])))
+      except TypeError,err:
+         submissionScript = ""
+         self.logger.log(logging.ERROR,getLogMessage("Submission script substitution failed:\n%s\n" % (rawSubmissionScript)))
+
+      return(submissionScript)
 
 
    def __buildCatalogs(self):
-      # setup regex's for the template
-      re_runName           = re.compile("RUNNAME")
-      re_jobId             = re.compile("JOBID")
-      re_instanceId        = re.compile("INSTANCEID")
-      re_instanceDirectory = re.compile("INSTANCEDIRECTORY")
-      re_scratchDirectory  = re.compile("SCRATCHDIRECTORY")
-      re_x509SubmitProxy   = re.compile("X509SUBMITPROXY")
-      re_pegasusHome       = re.compile("PEGASUSHOME")
+      substitutions = {}
+      substitutions["SUBMITTERCLASS"]       = str(self.submitterClass)
+      substitutions["RUNNAME"]              = self.runName
+      substitutions["JOBID"]                = self.localJobId
+      substitutions["INSTANCEID"]           = self.instanceId
+      substitutions["INSTANCEDIRECTORY"]    = self.instanceDirectory
+      substitutions["BASESCRATCHDIRECTORY"] = os.path.basename(self.scratchDirectory)
+      substitutions["SCRATCHDIRECTORY"]     = self.scratchDirectory
+      if self.x509SubmitProxy != "":
+         substitutions["X509SUBMITPROXY"]   = self.x509SubmitProxy
+      else:
+         substitutions["X509SUBMITPROXY"]   = os.path.join(os.sep,'tmp','hub-proxy.%s' % (self.hubUserName))
+      substitutions["PEGASUSHOME"]          = self.pegasusHome
+      substitutions["SSHPRIVATEKEYPATH"]    = self.sshIdentityPath
 
       for templateType in self.pegasusTemplates:
          pegasusTemplate = self.pegasusTemplates[templateType]
          if pegasusTemplate != "":
-            fpTemplate = open(pegasusTemplate,'r')
-            if fpTemplate:
-               template = ''.join(fpTemplate.readlines())
-               fpTemplate.close()
-               template = re_jobId.sub(self.localJobId,template)
-               template = re_instanceId.sub(self.instanceId,template)
-               template = re_instanceDirectory.sub(self.instanceDirectory,template)
-               template = re_scratchDirectory.sub(self.scratchDirectory,template)
-               if self.x509SubmitProxy != "":
-                  template = re_x509SubmitProxy.sub(self.x509SubmitProxy,template)
+            try:
+               fpTemplate = open(pegasusTemplate,'r')
+               try:
+                  template = ''.join(fpTemplate.readlines())
+               except (IOError,OSError):
+                  self.logger.log(logging.ERROR,getLogMessage("%s could not be read" % (pegasusTemplate)))
                else:
-                  template = re_x509SubmitProxy.sub(os.path.join(os.sep,'tmp','hub-proxy.%s' % (self.hubUserName)),template)
-               template = re_pegasusHome.sub(self.pegasusHome,template)
-               if   templateType == 'rc':
-                  pegasusFile = "%s_%s.pegasusrc" % (self.localJobId,self.instanceId)
-               elif templateType == 'sites':
-                  pegasusFile = "%s_%s_sites.xml" % (self.localJobId,self.instanceId)
-               elif templateType == 'tc':
-                  pegasusFile = "%s_%s_tc.txt" % (self.localJobId,self.instanceId)
-               pegasusPath = os.path.join(self.instanceDirectory,pegasusFile)
-               fpPegasusFile = open(pegasusPath,'w')
-               if fpPegasusFile:
-                  fpPegasusFile.write(template)
-                  fpPegasusFile.close()
+                  pegasusTemplate = ParameterTemplate(template)
+                  try:
+                     pegasusText = pegasusTemplate.substitute_recur(substitutions)
+                  except KeyError,err:
+                     pegasusText = ""
+                     self.logger.log(logging.ERROR,getLogMessage("Pattern substitution failed for @@%s\n" % (err[0])))
+                  except TypeError,err:
+                     pegasusText = ""
+                     self.logger.log(logging.ERROR,getLogMessage("Pegasus template substitution failed:\n%s\n" % (template)))
+
+                  if   templateType == 'rc':
+                     pegasusFile = "%s_%s.pegasusrc" % (self.localJobId,self.instanceId)
+                  elif templateType == 'sites':
+                     pegasusFile = "%s_%s_sites.xml" % (self.localJobId,self.instanceId)
+                  elif templateType == 'tc':
+                     pegasusFile = "%s_%s_tc.txt" % (self.localJobId,self.instanceId)
+                  pegasusPath = os.path.join(self.instanceDirectory,pegasusFile)
+                  try:
+                     fpPegasusFile = open(pegasusPath,'w')
+                     try:
+                        fpPegasusFile.write(pegasusText)
+                     except (IOError,OSError):
+                        self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (pegasusPath)))
+                     finally:
+                        fpPegasusFile.close()
+                  except (IOError,OSError):
+                     self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (pegasusPath)))
+               finally:
+                  fpTemplate.close()
+            except (IOError,OSError):
+               self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (pegasusTemplate)))
 
 
    def __buildDAX(self):
@@ -358,14 +343,22 @@ exit ${exitStatus}
 
          jobId = "%s_0" % (self.runName)
          job = Job(name=self.appScriptName,id=jobId)
+         job.addProfile(Profile(Namespace.HINTS, key='executionPool',value="%s" % (self.gridsite.split(',')[0])))
+         job.addProfile(Profile(Namespace.GLOBUS,key='maxwalltime',value=self.wallTime))
          job.uses(inputTar,link=Link.INPUT,executable=False)
          job.uses(outputTar,link=Link.OUTPUT,register=True,transfer=True)
          dax.addJob(job)
 
-         fpDAX = open(self.daxPath,'w')
-         if fpDAX:
-            dax.writeXML(fpDAX)
-            fpDAX.close()
+         try:
+            fpDAX = open(self.daxPath,'w')
+            try:
+               dax.writeXML(fpDAX)
+            except (IOError,OSError):
+               self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (self.daxPath)))
+            finally:
+               fpDAX.close()
+         except (IOError,OSError):
+            self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (self.daxPath)))
 
 
    def buildBatchScript(self):
@@ -375,11 +368,12 @@ exit ${exitStatus}
          batchScript = ""
       else:
          batchScript = self.__buildSerialFile()
+      batchScriptExecutable = True
 
       self.__buildCatalogs()
       self.__buildDAX()
 
-      return(batchLogName,batchScriptName,batchScript)
+      return(batchLogName,batchScriptName,batchScript,batchScriptExecutable)
 
 
    def getBatchNodeList(self):
@@ -394,3 +388,14 @@ exit ${exitStatus}
       return(daxExecutables)
 
 
+   def getRemoteJobIdNumber(self,
+                            remoteJobId):
+      try:
+# 1 job(s) submitted to cluster 105.
+         remoteJobIdNumber = re.search('cluster [0-9]+',remoteJobId).group().split()[1] + ".0"
+      except:
+         remoteJobIdNumber = "-1"
+
+      return(remoteJobIdNumber)
+
+
