@@ -1,30 +1,31 @@
 # @package      hubzero-submit-distributor
 # @file         VenueMechanismCore.py
 # @author       Steven Clark <clarks@purdue.edu>
-# @copyright    Copyright (c) 2012-2014 HUBzero Foundation, LLC.
-# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
+# @copyright    Copyright (c) 2012-2015 HUBzero Foundation, LLC.
+# @license      http://opensource.org/licenses/MIT MIT
 #
-# Copyright (c) 2012-2014 HUBzero Foundation, LLC.
+# Copyright (c) 2012-2015 HUBzero Foundation, LLC.
 #
-# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
-# software: you can redistribute it and/or modify it under the terms of
-# the GNU Lesser General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any
-# later version.
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 #
-# HUBzero is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
 #
 # HUBzero is a registered trademark of HUBzero Foundation, LLC.
 #
-
 import sys
 import os
 import select
@@ -45,23 +46,29 @@ class VenueMechanismCore:
    def __init__(self,
                 timeHistoryLogs,
                 siteInfo,
+                submissionScriptsInfo,
                 managerInfo,
                 remoteMonitors,
                 isMultiCoreRequest,
                 nCpus,
                 nNodes,
                 ppn):
-      self.logger              = logging.getLogger(__name__)
-      self.timeHistoryLogs     = copy.copy(timeHistoryLogs)
-      self.siteInfo            = copy.copy(siteInfo)
-      self.managerInfo         = copy.copy(managerInfo)
-      self.remoteJobMonitor    = remoteMonitors['job']
-      self.remoteTunnelMonitor = remoteMonitors['tunnel']
-      self.remoteCloudMonitor  = remoteMonitors['cloud']
+      self.logger                = logging.getLogger(__name__)
+      self.timeHistoryLogs       = copy.copy(timeHistoryLogs)
+      self.siteInfo              = copy.copy(siteInfo)
+      self.submissionScriptsInfo = copy.copy(submissionScriptsInfo)
+      self.managerInfo           = copy.copy(managerInfo)
+      self.remoteJobMonitor      = remoteMonitors['job']
+      self.remoteTunnelMonitor   = remoteMonitors['tunnel']
+      self.remoteCloudMonitor    = remoteMonitors['cloud']
+      self.remoteIdentityManager = remoteMonitors['identity']
 
       self.childPid   = 0
       self.bufferSize = 4096
 
+      self.remoteAppScript = None
+      self.remoteBatch     = None
+
       self.venueMechanism    = ''
       self.remoteBatchSystem = ''
       self.enteredCommand    = ""
@@ -127,23 +134,31 @@ class VenueMechanismCore:
             if not jobIndex in self.jobStatistics:
                self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
 
-            fpGridResource = open(os.path.join(self.instanceDirectory,matchingFile),'r')
-            if fpGridResource:
-               gridResource = fpGridResource.readline().strip()
-               fpGridResource.close()
-               gridType = gridResource.split()[0]
-               if   gridType == 'gt2':
-                  gateKeeper = gridResource.split()[1].split('/')[0]
-                  jobManager = gridResource.split()[1].split('/')[1].split('-')[1].upper()
-               elif gridType == 'gt4':
-                  gateKeeper = gridResource.split()[1]
-                  jobManager = gridResource.split()[2]
-               elif gridType == 'gt5':
-                  gateKeeper = gridResource.split()[1].split('/')[0]
-                  jobManager = gridResource.split()[1].split('/')[1].split('-')[1].upper()
+            try:
+               gridResourcePath = os.path.join(self.instanceDirectory,matchingFile)
+               fpGridResource = open(gridResourcePath,'r')
+               try:
+                  gridResource = fpGridResource.readline().strip()
+               except (IOError,OSError):
+                  self.logger.log(logging.ERROR,getLogMessage("%s could not be read" % (gridResourcePath)))
+               else:
+                  gridType = gridResource.split()[0]
+                  if   gridType == 'gt2':
+                     gateKeeper = gridResource.split()[1].split('/')[0]
+                     jobManager = gridResource.split()[1].split('/')[1].split('-')[1].upper()
+                  elif gridType == 'gt4':
+                     gateKeeper = gridResource.split()[1]
+                     jobManager = gridResource.split()[2]
+                  elif gridType == 'gt5':
+                     gateKeeper = gridResource.split()[1].split('/')[0]
+                     jobManager = gridResource.split()[1].split('/')[1].split('-')[1].upper()
 
-               self.jobStatistics[jobIndex]['venue']                  = gateKeeper
-               self.jobStatistics[jobIndex]['jobSubmissionMechanism'] = self.venueMechanism + jobManager
+                  self.jobStatistics[jobIndex]['venue']                  = gateKeeper
+                  self.jobStatistics[jobIndex]['jobSubmissionMechanism'] = self.venueMechanism + jobManager
+               finally:
+                  fpGridResource.close()
+            except (IOError,OSError):
+               self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (gridResourcePath)))
 
             os.remove(os.path.join(self.instanceDirectory,matchingFile))
       else:
@@ -153,7 +168,12 @@ class VenueMechanismCore:
 
    def logGridHistories(self):
 
-      fpGridHistoryLog = open(self.timeHistoryLogs['hubGridJobHistoryLogPath'],'a')
+      gridHistoryLogPath = self.timeHistoryLogs['hubGridJobHistoryLogPath']
+      try:
+         fpGridHistoryLog = open(gridHistoryLogPath,'a')
+      except (IOError,OSError):
+         fpGridHistoryLog = None
+         self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (gridHistoryLogPath)))
 
       reFiles = re.compile(self.timeHistoryLogs['jobGridHistory'] + "_[0-9]+$")
       dirFiles = os.listdir(self.instanceDirectory)
@@ -169,14 +189,25 @@ class VenueMechanismCore:
             if not jobIndex in self.jobStatistics:
                self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
 
+            gridHistoryPath = os.path.join(self.instanceDirectory,matchingFile)
             if fpGridHistoryLog:
-               fpGridHistory = open(os.path.join(self.instanceDirectory,matchingFile),'r')
-               if fpGridHistory:
-                  gridHistory = fpGridHistory.readline()
-                  fpGridHistoryLog.write("%s_%s\t%d\t%s\t%s\n" % (self.localJobId,self.instanceId,jobIndex, \
-                                                                  gridHistory.strip(),self.enteredCommand))
-                  fpGridHistory.close()
-            os.remove(os.path.join(self.instanceDirectory,matchingFile))
+               try:
+                  fpGridHistory = open(gridHistoryPath,'r')
+                  try:
+                     gridHistory = fpGridHistory.readline()
+                  except (IOError,OSError):
+                     self.logger.log(logging.ERROR,getLogMessage("%s could not be read" % (gridHistoryPath)))
+                  else:
+                     try:
+                        fpGridHistoryLog.write("%s_%s\t%d\t%s\t%s\n" % (self.localJobId,self.instanceId,jobIndex, \
+                                                                        gridHistory.strip(),self.enteredCommand))
+                     except (IOError,OSError):
+                        self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (gridHistoryLogPath)))
+                  finally:
+                     fpGridHistory.close()
+               except (IOError,OSError):
+                  self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (gridHistoryPath)))
+            os.remove(gridHistoryPath)
 
       reFiles = re.compile(self.timeHistoryLogs['jobGridHistory'] + "_[0-9]+_$")
       dirFiles = os.listdir(self.instanceDirectory)
@@ -192,18 +223,31 @@ class VenueMechanismCore:
             if not jobIndex in self.jobStatistics:
                self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
 
+            gridHistoryPath = os.path.join(self.instanceDirectory,matchingFile)
             if fpGridHistoryLog:
-               fpGridHistory = open(os.path.join(self.instanceDirectory,matchingFile),'r')
-               if fpGridHistory:
-                  jobInProgressSite,jobInProgressSetupCompleted = fpGridHistory.readline().rstrip().split()
-                  jobInProgressCompleted = int(time.mktime(datetime.datetime.utcnow().timetuple()))
-                  jobInProgressExitStatus = -3
-                  fpGridHistoryLog.write("%s_%s\t%d\t%s\t%s\t%d\t%d\t%s\n" % (self.localJobId,self.instanceId,jobIndex, \
-                                                                              jobInProgressSite,jobInProgressSetupCompleted, \
-                                                                              jobInProgressCompleted,jobInProgressExitStatus, \
-                                                                              self.enteredCommand))
-                  fpGridHistory.close()
-            os.remove(os.path.join(self.instanceDirectory,matchingFile))
+               try:
+                  fpGridHistory = open(gridHistoryPath,'r')
+                  try:
+                     jobInProgressSite,jobInProgressSetupCompleted = fpGridHistory.readline().rstrip().split()
+                  except (IOError,OSError):
+                     self.logger.log(logging.ERROR,getLogMessage("%s could not be read" % (gridHistoryPath)))
+                  else:
+                     jobInProgressCompleted = int(time.mktime(datetime.datetime.utcnow().timetuple()))
+                     jobInProgressExitStatus = -3
+                     try:
+                        fpGridHistoryLog.write("%s_%s\t%d\t%s\t%s\t%d\t%d\t%s\n" % (self.localJobId,self.instanceId,jobIndex, \
+                                                                                    jobInProgressSite, \
+                                                                                    jobInProgressSetupCompleted, \
+                                                                                    jobInProgressCompleted,
+                                                                                    jobInProgressExitStatus, \
+                                                                                    self.enteredCommand))
+                     except (IOError,OSError):
+                        self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (gridHistoryLogPath)))
+                  finally:
+                     fpGridHistory.close()
+               except (IOError,OSError):
+                  self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (gridHistoryPath)))
+            os.remove(gridHistoryPath)
 
       if fpGridHistoryLog:
          fpGridHistoryLog.close()
@@ -214,7 +258,13 @@ class VenueMechanismCore:
       dirFiles = os.listdir(self.instanceDirectory)
       matchingFiles = filter(reFiles.search,dirFiles)
       if len(matchingFiles) > 0:
-         fpGridJobIdLog = open(self.timeHistoryLogs['hubGridJobIdLogPath'],'a')
+         gridJobIdLogPath = self.timeHistoryLogs['hubGridJobIdLogPath']
+         try:
+            fpGridJobIdLog = open(gridJobIdLogPath,'a')
+         except (IOError,OSError):
+            fpGridJobIdLog = None
+            self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (gridJobIdLogPath)))
+
          matchingFiles.sort()
          for matchingFile in matchingFiles:
             if len(matchingFile) > len(self.timeHistoryLogs['jobGridJobId']):
@@ -225,13 +275,24 @@ class VenueMechanismCore:
             if not jobIndex in self.jobStatistics:
                self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
 
+            gridJobIdPath = os.path.join(self.instanceDirectory,matchingFile)
             if fpGridJobIdLog:
-               fpGridJobId = open(os.path.join(self.instanceDirectory,matchingFile),'r')
-               if fpGridJobId:
-                  gridJobId = fpGridJobId.readline()
-                  fpGridJobIdLog.write("%s_%s %s\n" % (self.localJobId.lstrip('0'),self.instanceId,gridJobId.strip()))
-                  fpGridJobId.close()
-            os.remove(os.path.join(self.instanceDirectory,matchingFile))
+               try:
+                  fpGridJobId = open(gridJobIdPath,'r')
+                  try:
+                     gridJobId = fpGridJobId.readline()
+                  except (IOError,OSError):
+                     self.logger.log(logging.ERROR,getLogMessage("%s could not be read" % (gridJobIdPath)))
+                  else:
+                     try:
+                        fpGridJobIdLog.write("%s_%s %s\n" % (self.localJobId.lstrip('0'),self.instanceId,gridJobId.strip()))
+                     except (IOError,OSError):
+                        self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (gridJobIdLogPath)))
+                  finally:
+                     fpGridJobId.close()
+               except (IOError,OSError):
+                  self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (gridJobIdPath)))
+            os.remove(gridJobIdPath)
          if fpGridJobIdLog:
             fpGridJobIdLog.close()
       else:
@@ -391,138 +452,30 @@ class VenueMechanismCore:
    def recordJobStatisticTime(self,
                               statistic,
                               timeFileBasename):
-      reFiles = re.compile(timeFileBasename + "(_[0-9]+)?$")
-      dirFiles = os.listdir(self.instanceDirectory)
-      matchingFiles = filter(reFiles.search,dirFiles)
-      if len(matchingFiles) > 0:
-         foundIndexes = []
-         matchingFiles.sort()
-         for matchingFile in matchingFiles:
-            if len(matchingFile) > len(timeFileBasename):
-               jobIndex = int(matchingFile[len(timeFileBasename)+1:])
-            else:
-               jobIndex = 1
-            foundIndexes.append(jobIndex)
+      timePath = os.path.join(self.instanceDirectory,timeFileBasename)
+      if os.path.exists(timePath):
+         jobIndex = int(self.instanceId)
 
-            if not jobIndex in self.jobStatistics:
-               self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
+         if not jobIndex in self.jobStatistics:
+            self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
+            self.logger.log(logging.DEBUG,getLogMessage("recordJobStatisticTime: added missing jobIndex %d" % (jobIndex)))
 
-            if(os.path.getsize(os.path.join(self.instanceDirectory,matchingFile))):
-               fpTime = open(os.path.join(self.instanceDirectory,matchingFile),'r')
-               if fpTime:
-                  self.jobStatistics[jobIndex][statistic] = float(fpTime.readline())
-                  fpTime.close()
-            os.remove(os.path.join(self.instanceDirectory,matchingFile))
-         if self.remoteBatchSystem != 'PEGASUS':
-            maximumJobIndex = max(self.jobStatistics.keys())
-            if len(foundIndexes) != maximumJobIndex:
-               for missingIndex in set(range(1,maximumJobIndex+1)).difference(set(foundIndexes)):
-                  self.logger.log(logging.ERROR,getLogMessage(timeFileBasename + '_' + str(missingIndex) + " is missing"))
-         del foundIndexes
+         self.jobStatistics[jobIndex].recordTime(statistic,timePath)
       else:
          self.logger.log(logging.ERROR,getLogMessage(timeFileBasename + " not present in " + self.instanceDirectory))
 
 
    def recordJobStatisticTimer(self,
                                timeFileBasename):
-      reFiles = re.compile(timeFileBasename + "(_[0-9]+)?$")
-      dirFiles = os.listdir(self.instanceDirectory)
-      matchingFiles = filter(reFiles.search,dirFiles)
-      if len(matchingFiles) > 0:
-         foundIndexes = []
-         matchingFiles.sort()
-         for matchingFile in matchingFiles:
-            if len(matchingFile) > len(timeFileBasename):
-               jobIndex = int(matchingFile[len(timeFileBasename)+1:])
-            else:
-               jobIndex = 1
-            foundIndexes.append(jobIndex)
+      timerPath = os.path.join(self.instanceDirectory,timeFileBasename)
+      if os.path.exists(timerPath):
+         jobIndex = int(self.instanceId)
 
-            if not jobIndex in self.jobStatistics:
-               self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
+         if not jobIndex in self.jobStatistics:
+            self.jobStatistics[jobIndex] = JobStatistic(self.nCpus)
+            self.logger.log(logging.DEBUG,getLogMessage("recordJobStatisticTimer: added missing jobIndex %d" % (jobIndex)))
 
-            venue       = None
-            jobId       = None
-            event       = None
-            realTime    = 0.
-            userTime    = 0.
-            sysTime     = 0.
-            elapsedTime = 0.
-            waitTime    = 0.
-            exitStatus  = 0
-            exitSignal  = 0
-
-            fpTimer = open(os.path.join(self.instanceDirectory,matchingFile),'r')
-            if fpTimer:
-               while 1:
-                  line = fpTimer.readline()
-                  if not line:
-                     break
-                  line = line.strip()
-
-                  if line != "":
-                     parts = line.split()
-                     if len(parts) == 2:
-                        timeType = parts[0]
-                        try:
-                           timeUsed = float(parts[1])
-
-                           if   timeType == 'real':
-                              realTime    = max(realTime,timeUsed)
-                           elif timeType == 'user':
-                              userTime    = userTime + timeUsed
-                           elif timeType == 'sys':
-                              sysTime     = sysTime + timeUsed
-                           elif timeType == 'elapsed':
-                              elapsedTime = elapsedTime + timeUsed
-                           elif timeType == 'wait':
-                              waitTime    = waitTime + timeUsed
-                           elif timeType == 'jobId':
-                              jobId       = parts[1]
-                        except:
-                           if   timeType == 'site':
-                              venue = parts[1]
-                           elif timeType == 'jobId':
-                              jobId = parts[1]
-                           elif timeType == 'event':
-                              event = parts[1]
-                     else:
-                        if len(parts) > 2:
-                           if parts[-2] == "status":
-                              exitStatus = int(float(parts[-1]))
-                           if parts[-2] == "signal":
-# Killed by signal 2.
-                              exitStatus = 1
-                              exitSignal = int(float(parts[-1]))
-                        if re.search("Killed",line):
-                           exitStatus = 1
-                           exitSignal = 15
-
-               if self.jobStatistics[jobIndex]['exitCode'] == 2:
-                  if exitSignal > 0:
-                     self.jobStatistics[jobIndex]['exitCode'] = exitStatus << 7 | exitSignal
-                  else:
-                     self.jobStatistics[jobIndex]['exitCode'] = exitStatus
-               self.jobStatistics[jobIndex]['realTime']       = realTime
-               self.jobStatistics[jobIndex]['userTime']       = userTime
-               self.jobStatistics[jobIndex]['sysTime']        = sysTime
-               self.jobStatistics[jobIndex]['elapsedRunTime'] = elapsedTime
-               self.jobStatistics[jobIndex]['waitingTime']    = waitTime
-               if venue and not self.jobStatistics[jobIndex]['venue']:
-                  self.jobStatistics[jobIndex]['venue']             = venue
-               if jobId:
-                  self.jobStatistics[jobIndex]['remoteJobIdNumber'] = jobId
-               if event:
-                  self.jobStatistics[jobIndex]['event']             = event
-
-               fpTimer.close()
-            os.remove(os.path.join(self.instanceDirectory,matchingFile))
-         if self.remoteBatchSystem != 'PEGASUS':
-            maximumJobIndex = max(self.jobStatistics.keys())
-            if len(foundIndexes) != maximumJobIndex:
-               for missingIndex in set(range(1,maximumJobIndex+1)).difference(set(foundIndexes)):
-                  self.logger.log(logging.ERROR,getLogMessage(timeFileBasename + '_' + str(missingIndex) + " is missing"))
-         del foundIndexes
+         self.jobStatistics[jobIndex].recordTimer(timerPath)
       else:
          if self.jobSubmitted:
             self.logger.log(logging.ERROR,getLogMessage(timeFileBasename + " not present in " + self.instanceDirectory))
@@ -575,15 +528,37 @@ class VenueMechanismCore:
                      os.rename(srcPath,dstPath)
 
 
+   def registerJob(self):
+      self.remoteJobMonitor.postJobRegistration(self.siteInfo['siteName'],
+                                                self.localJobId,self.instanceId,
+                                                str(self.hubUserId),self.submitterClass,
+                                                self.distributorPid)
+
+
+   def isJobReleased(self):
+      jobReleased = self.remoteJobMonitor.isJobReleased(self.localJobId,self.instanceId)
+
+      return(jobReleased)
+
+
+   def removeJobRegistration(self):
+      self.remoteJobMonitor.deleteJobRegistration(self.localJobId,self.instanceId)
+
+
    def removeInstanceDirectory(self):
-      if self.instanceDirectory != os.getcwd():
-         shutil.rmtree(self.instanceDirectory,True)
-         jobDirectory = os.path.dirname(self.instanceDirectory)
-         if jobDirectory != os.getcwd():
-            try:
-               os.rmdir(jobDirectory)
-            except:
-               pass
+      try:
+         currentWorkingDirectory = os.getcwd()
+      except (IOError,OSError):
+         self.logger.log(logging.ERROR,getLogMessage("removeInstanceDirectory:os.getcwd(): No such file or directory"))
+      else:
+         if self.instanceDirectory != currentWorkingDirectory:
+            shutil.rmtree(self.instanceDirectory,True)
+            jobDirectory = os.path.dirname(self.instanceDirectory)
+            if jobDirectory != currentWorkingDirectory:
+               try:
+                  os.rmdir(jobDirectory)
+               except:
+                  pass
 
 
    def moveTree(self,
@@ -617,41 +592,69 @@ class VenueMechanismCore:
 
    def recoverFiles(self,
                     ingoreInstanceFiles):
-      if self.instanceDirectory != os.getcwd():
+      try:
+         currentWorkingDirectory = os.getcwd()
+      except (IOError,OSError):
+         self.logger.log(logging.ERROR,getLogMessage("recoverFiles:os.getcwd(): No such file or directory"))
+      else:
          if os.path.exists(self.instanceDirectory):
-            jobDirectory = os.getcwd()
-            dirFiles = os.listdir(self.instanceDirectory)
-            if ingoreInstanceFiles:
-               reIgnoreFiles = re.compile(".*%s_%s.*" % (self.runName,self.instanceId))
-               ignoreFiles = filter(reIgnoreFiles.search,dirFiles)
-               for ignoreFile in ignoreFiles:
-                  dirFiles.remove(ignoreFile)
+            if os.path.isdir(self.instanceDirectory):
+               if self.instanceDirectory != currentWorkingDirectory:
+                  dirFiles = os.listdir(self.instanceDirectory)
+                  if ingoreInstanceFiles:
+                     reIgnoreFiles = re.compile(".*%s_%s.*" % (self.runName,self.instanceId))
+                     ignoreFiles = filter(reIgnoreFiles.search,dirFiles)
+                     for ignoreFile in ignoreFiles:
+                        dirFiles.remove(ignoreFile)
 
-            for dirFile in dirFiles:
-               srcFile = os.path.join(self.instanceDirectory,dirFile)
-               dstFile = os.path.join(jobDirectory,dirFile)
-               try:
-                  if os.path.isdir(srcFile):
-                     self.moveTree(srcFile,dstFile)
-                  else:
-                     os.rename(srcFile,dstFile)
-               except:
-                  self.logger.log(logging.ERROR,getLogMessage("%s recovery failed" % (srcFile)))
+                  for dirFile in dirFiles:
+                     srcFile = os.path.join(self.instanceDirectory,dirFile)
+                     dstFile = os.path.join(currentWorkingDirectory,dirFile)
+                     try:
+                        if os.path.isdir(srcFile):
+                           self.moveTree(srcFile,dstFile)
+                        else:
+                           os.rename(srcFile,dstFile)
+                     except:
+                        self.logger.log(logging.ERROR,getLogMessage("%s recovery failed" % (srcFile)))
 
 
    def recoverStdFiles(self):
-      if self.instanceDirectory != os.getcwd():
-         for fileType in 'stderr','stdout','FAILURE':
-            fileToRecover = os.path.join(self.instanceDirectory,"%s_%s.%s" % (self.runName,self.instanceId,fileType))
-            if os.path.exists(fileToRecover):
-               fileToAppend = "%s.%s" % (self.runName,fileType)
-               fpAppend = open(fileToAppend,'a')
-               if fpAppend:
-                  fpRecover = open(fileToRecover,'r')
-                  if fpRecover:
-                     recoverText = fpRecover.readlines()
-                     fpRecover.close()
-                     fpAppend.writelines(recoverText)
-                  fpAppend.close()
+      try:
+         currentWorkingDirectory = os.getcwd()
+      except (IOError,OSError):
+         self.logger.log(logging.ERROR,getLogMessage("recoverStdFiles:os.getcwd(): No such file or directory"))
+      else:
+         if os.path.isdir(self.instanceDirectory):
+            if self.instanceDirectory != currentWorkingDirectory:
+               for fileType in 'stderr','stdout','FAILURE':
+                  reFiles = re.compile(".*%s_%s\.%s" % (self.runName,self.instanceId,fileType))
+                  dirFiles = os.listdir(self.instanceDirectory)
+                  matchingFiles = filter(reFiles.search,dirFiles)
+                  if len(matchingFiles) > 0:
+                     matchingFiles.sort(reverse=True)
+                     fileToAppend = "%s.%s" % (self.runName,fileType)
+                     try:
+                        fpAppend = open(fileToAppend,'a')
+                        for matchingFile in matchingFiles:
+                           fileToRecover = os.path.join(self.instanceDirectory,matchingFile)
+                           try:
+                              fpRecover = open(fileToRecover,'r')
+                              try:
+                                 recoverText = fpRecover.readlines()
+                              except (IOError,OSError):
+                                 self.logger.log(logging.ERROR,getLogMessage("%s could not be read" % (fileToRecover)))
+                              else:
+                                 try:
+                                    fpAppend.writelines(recoverText)
+                                 except (IOError,OSError):
+                                    self.logger.log(logging.ERROR,getLogMessage("%s could not be written" % (fileToAppend)))
+                              finally:
+                                 fpRecover.close()
+                           except (IOError,OSError):
+                              self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (fileToRecover)))
+                        fpAppend.close()
+                     except (IOError,OSError):
+                        self.logger.log(logging.ERROR,getLogMessage("%s could not be opened" % (fileToAppend)))
 
 
